import os
from unittest.mock import mock_open, patch

from casa_envoy.launchers import CalibrationLauncher, ImagingLauncher, CasaLauncher

cal_parameters = {
    "useCasa": False,
    "homeForReprocessing": "/home/casa/packages/pipeline/current",
    "rootDirectory": "/tmp/workspaces_tmp/",
    "processingDirectory": "tmpiox5trbp",
    "metadata": "test/test.json",
    "ppr": "test/PPR.xml",
    "product_type": "standard-cal",
}
img_parameters = {
    "useCasa": False,
    "homeForReprocessing": "/home/casa/packages/pipeline/current",
    "rootDirectory": "/tmp/workspaces_tmp/",
    "processingDirectory": "tmpiox5trbp",
    "metadata": "test/image-metadata.json",
    "ppr": "test/cmsimage-PPR.xml",
    "product_type": "standard-img",
}


class TestCasaLauncher:
    def test_setup_environment(self):
        CasaLauncher(parameters=cal_parameters).setup_environment()
        assert os.environ.get("SCIPIPE_ROOTDIR") == cal_parameters["rootDirectory"]
        assert os.environ.get("CASA_HOME") == cal_parameters["homeForReprocessing"]
        assert os.environ.get("PPR_FILENAME") == "test/PPR.xml"

    @patch("subprocess.run")
    @patch("os.chdir")
    def test_run(self, mock_os, mock_subprocess):
        CasaLauncher(parameters=cal_parameters).run()
        assert mock_subprocess.call_count == 1

    @patch("builtins.open")
    @patch("glob.glob")
    @patch("os.chdir")
    @patch("os.getcwd")
    def test_check_logs(self, mock_os_cwd, mock_os_dir, mock_glob, mock_open):
        CasaLauncher(parameters=cal_parameters).check_logs()
        assert mock_os_cwd.call_count == 1
        assert mock_os_dir.call_count == 0
        assert mock_glob.call_count == 1


class TestCalibrationLauncher:
    @patch("casa_envoy.launchers.CalibrationLauncher.run_audit")
    @patch("casa_envoy.launchers.CasaLauncher.run")
    def test_launch_casa(self, mock_run, mock_audit):
        CalibrationLauncher(parameters=cal_parameters).launch_casa()
        assert mock_run.call_count == 1
        assert mock_audit.call_count == 1

    def test_check_calibratable(self):
        check = CalibrationLauncher(parameters=cal_parameters).check_calibratable()
        assert check is True


class TestImagingLauncher:
    @patch("casa_envoy.launchers.CalibrationLauncher.run_audit")
    @patch("casa_envoy.launchers.CasaLauncher.run")
    def test_launch_casa(self, mock_run, mock_audit):
        CalibrationLauncher(parameters=img_parameters).launch_casa()
        assert mock_run.call_count == 1
        assert mock_audit.call_count == 1

    def test_check_imageable(self):
        check = ImagingLauncher(parameters=img_parameters).check_imageable()
        assert check is True
