"""
Tests for conveyor.deliver module
"""
from unittest.mock import patch, MagicMock

from conveyor.deliver import DeliveryConveyor

settings = {
    "qa_delivery_area": "/lustre/aoc/cluster/pipeline/docker/workspaces/qa2",
    "weblog_cache_area": "/lustre/aoc/cluster/pipeline/docker/workspaces/cache/weblog",
    "workspaces_lustre_root_dir": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool",
    "current_root_directory": "current/directory",
    "current_subdirectory": "dir_123",
    "destination_dir": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool/tmpabcd1234",
    "destination_subdir": "tmpabcd1234",
}

action = "Calibration Delivery"

conveyor = DeliveryConveyor(settings=settings, action=action)


class TestDeliveryConveyor:
    def test_delivery(self):
        with patch("conveyor.deliver.DeliveryConveyor.cache_weblog") as weblog:
            with patch("conveyor.deliver.DeliveryConveyor.move_subdirectories_to_qa_area") as qa2:
                conveyor.delivery()
                assert weblog.call_count == 1
                assert qa2.call_count == 1

    @patch("sys.exit")
    @patch("shutil.copytree")
    @patch("tarfile.open")
    def test_cache_weblog(self, mock_tar, mock_shutil, mock_exit):
        with patch("os.makedirs") as make_dirs:
            with patch("conveyor.deliver.DeliveryConveyor._get_pipeline_dir") as pipeline:
                conveyor.cache_weblog()
                assert make_dirs.call_count == 1
                assert mock_shutil.call_count == 1

    def test_get_pipeline_dir(self):
        with patch("glob.glob") as glob:
            choice = conveyor._get_pipeline_dir()
            assert glob.call_count == 1
            assert choice is True

    @patch("os.symlink")
    @patch("os.system")
    @patch("shutil.copytree")
    @patch("os.chown")
    @patch("os.makedirs")
    def test_move_subdirectories_to_qa_area(
        self, mock_dirs, mock_chown, mock_copy, mock_system, mock_link
    ):
        with patch(
            "glob.glob", MagicMock(return_value=["rawdata/", "working/", "products/"])
        ) as contents:
            path = "/lustre/aoc/cluster/pipeline/docker/workspaces/qa2/tmpabcd1234"
            conveyor.move_subdirectories_to_qa_area(path)
            assert mock_dirs.call_count == 1
            assert mock_chown.call_count == 1
            assert mock_copy.call_count == 3
            assert mock_system.call_count == 3
            assert mock_link.call_count == 3

    @patch("os.symlink")
    def symlink_qa(self, mock_link):
        conveyor.symlink_qa(settings["destination_subdir"], settings["qa_delivery_area"])
        assert mock_link.call_count == 1

    def test_convey(self):
        with patch("conveyor.deliver.DeliveryConveyor.delivery") as delivery:
            conveyor.convey()
            assert delivery.call_count == 1
